# Edit this configuration file to define what should be installed on
# your system.  Help is available in the configuration.nix(5) man page
# and in the NixOS manual (accessible by running `nixos-help`).

{ config, pkgs, ... }:
let
  # Import Home Manager with niv.
  home-manager = (import ../../nix/sources.nix).home-manager;
  #home-manager-source = import (import ../../nix/sources.nix).home-manager {};
in
{
  # Paths to other modules.
  # Compose this module out of smaller ones.
  # https://nixos.wiki/wiki/NixOS_modules
  imports =
    [
      # Include the results of the hardware scan.
      ./hardware-configuration.nix
      # Home Manager.
      (import "${home-manager}/nixos")
      #home-manager-source.nixos
      ../../modules/home-manager.nix
      # NUR.
      ../../modules/nur.nix
      # Device specific.
      #../../modules/yodaNas.nix

      ../../modules/base.nix
      ../../modules/base-headless.nix
      #../../modules/base-gnome.nix
      #../../modules/programs.nix
      #../../modules/autostart.nix
      #../../modules/wallpaper.nix
      #../../modules/fwupd-gnome.nix
      #../../modules/print-and-scan.nix
      ../../modules/fde-ssh-unlock.nix
      #../../modules/nextcloud-integration.nix

      #../../modules/gnome-config.nix
      #../../modules/gnome-extensions.nix
      #../../modules/gnome-fractional-scaling.nix
      #../../modules/git.nix
      ../../modules/zsh.nix
      #../../modules/nitrokey-gpg-smartcard.nix
      #../../modules/ssh-client.nix
      ../../modules/ssh-server.nix
      #../../modules/firefox.nix
      #../../modules/thunderbird.nix
      #../../modules/digikam-rawtherapee.nix
      #../../modules/android.nix
      #../../modules/podman.nix
      ../../modules/docker.nix
      #../../modules/docker-pushrm.nix
      #../../modules/lid-switch-handling.nix
      ../../modules/sendmail-mta.nix
      ../../modules/journalwatch.nix
      #../../modules/waydroid.nix
      #../../modules/ntfs.nix
     #../../modules/veracrypt.nix
    ];

  networking.hostName = "yodaNas";
  boot.initrd.luks.devices."luks-3d974bd0-f373-469b-8e9c-2d5516e9f0f5".allowDiscards = true;

  boot.kernelParams = [];

  boot.kernelPackages = pkgs.linuxPackages;

  # TODO: For "/" this cfg resides in base.nix
  fileSystems = {
    "/mnt/data".options   = [ "compress=zstd" "noatime" "commit=120" ];
    "/mnt/backup".options = [ "compress=zstd" "noatime" "commit=120" ];
  };
}
